﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Collections;

namespace HeuristicLab.Problems.MetaOptimization {
  // todo: check that at least 1 elements needs to be selected
  // todo: control creatable item types
  [StorableClass]
  public class CheckedValueConfigurationList : CheckedItemList<IValueConfiguration>, ICheckedValueConfigurationList {
    [Storable]
    private int minItemCount = 1;
    public int MinItemCount { 
      get { return minItemCount; } 
    }

    [Storable]
    protected IItemSet<IItem> validValues;
    public IItemSet<IItem> ValidValues {
      get { return validValues; }
    }

    public CheckedValueConfigurationList(IItemSet<IItem> validValues) {
      this.validValues = validValues;
      RegisterEvents();
    }
    public CheckedValueConfigurationList() {
      RegisterEvents();
    }
    [StorableConstructor]
    protected CheckedValueConfigurationList(bool deserializing) : base(deserializing) {
      RegisterEvents();
    }
    protected CheckedValueConfigurationList(CheckedValueConfigurationList original, Cloner cloner) : base(original, cloner) {
      this.minItemCount = original.MinItemCount;
      this.validValues = original.validValues;
      RegisterEvents();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CheckedValueConfigurationList(this, cloner);
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEvents();
    }

    private void RegisterEvents() {
      this.ItemsRemoved += new CollectionItemsChangedEventHandler<IndexedItem<IValueConfiguration>>(CheckedValueConfigurationList_ItemsRemoved);
    }

    private void DeregisterEvents() {
      this.ItemsRemoved -= new CollectionItemsChangedEventHandler<IndexedItem<IValueConfiguration>>(CheckedValueConfigurationList_ItemsRemoved);
    }

    private void CheckedValueConfigurationList_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<IndexedItem<IValueConfiguration>> e) {
      // auch collectionreset gehört berücksichtigt
      // funktioniert so nicht ganz, weil die view das hinzufügen nicht mitkriegt
      //if (this.Count < minItemCount) {
      //  foreach (var item in e.Items) {
      //    if (this.Count >= minItemCount)
      //      break;
      //    this.Add(item);
      //  }
      //}
    }

  }
}
